/*****************************************************************************/
/* File name: Entry.java                                                     */
/* Purpose: to keep mount entry tuple data:                                  */
/*          directory path, host name, file handle                           */
/*                                                                           */
/* Last modified: 21.06.99                                                   */
/* Author: Victor Klimov, 1999                                               */
/*                                                                           */
/* This program is distributed in the hope that it will be useful, but       */
/* WITHOUT ANY WARRANTY; without even the implied warranty of                */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General */
/* Public License for more details.                                          */
/*                                                                           */
/* This software is distributed under the terms                              */
/* of the GNU General Public License as published                            */
/* by the Free Software Foundation; either version 2, or (at your option)    */
/* any later version.                                                        */
/* See file COPYING for your rights (GNU GPL)                                */
/*****************************************************************************/

package mount;


import java.io.*;


public
class

Entry
       implements Serializable
{

  private final static String RCSID =
"$Header: /d/NFS/mount/RCS/Entry.java,v 1.2 1999/06/23 17:00:33 vklimov Exp $";


  protected Fhandle fileHandle = null;


  public

  Entry()
/*****************************************************************************/
/* Function name: Entry                                                      */
/* File name: Entry.java                                                     */
/* Purpose: constructor for the Entry class                                  */
/* Parameters:                                                               */
/* Returns:                                                                  */
/* Last modified: 13.06.99                                                   */
/* Author: Victor Klimov, 1999                                               */
/*****************************************************************************/
    {
                                                           /* Entry */
/*---------------------------------------------------------------------------*/

    }
/*end of Entry*/


  public

  Entry( Fhandle fileHandle )
/*****************************************************************************/
/* Function name: Entry                                                      */
/* File name: Entry.java                                                     */
/* Purpose: constructor for the Entry class                                  */
/* Parameters:                                                               */
/* Returns:                                                                  */
/* Last modified: 21.06.99                                                   */
/* Author: Victor Klimov, 1999                                               */
/*****************************************************************************/
    {
                                                          /* Entry */
/*---------------------------------------------------------------------------*/

      this.fileHandle = fileHandle;

    }
/*end of Entry*/


  public
  String

  toString()
/*****************************************************************************/
/* Function name: toString                                                   */
/* File name: Entry.java                                                     */
/* Purpose: to represent this Entry object as a string                       */
/* Parameters:                                                               */
/* Returns: the string representation of the object                          */
/* Last modified: 21.06.99                                                   */
/* Author: Victor Klimov, 1999                                               */
/*****************************************************************************/
    {
      String result;
                                                       /* toString */
/*---------------------------------------------------------------------------*/

      if (fileHandle == null)
        result = "fileHandle is null";
      else
        result =
          "fileHandle=\n" + fileHandle.toString();
    /*end of if*/

      return result;

    }
/*end of toString*/


  public
  int

  hashCode()
/*****************************************************************************/
/* Function name: hashCode                                                   */
/* File name: Entry.java                                                     */
/* Purpose: to return hash code (dirPath based) for this Entry object        */
/* Parameters:                                                               */
/* Returns: the hash code                                                    */
/* Last modified: 21.06.99                                                   */
/* Author: Victor Klimov, 1999                                               */
/*****************************************************************************/
    {
      int result;
                                                       /* hashCode */
/*---------------------------------------------------------------------------*/

      if (fileHandle == null)
        result = 0;
      else
        result = fileHandle.getDirPath().hashCode();
    /*end of if*/

      return result;

    }
/*end of hashCode*/


  public
  boolean

  equals( Entry mountEntry )
/*****************************************************************************/
/* Function name: equals                                                     */
/* File name: Entry.java                                                     */
/* Purpose: to check whether this Entry object is equal to the argument      */
/* Parameters: Entry                                                         */
/* Returns: true - if the object are equal, false - otherwise                */
/* Last modified: 21.06.99                                                   */
/* Author: Victor Klimov, 1999                                               */
/*****************************************************************************/
    {
                                                            /* equals */
/*---------------------------------------------------------------------------*/

      if (mountEntry == null)
        return false;                                   /* return ! */
      else
        ;
    /*end of if*/

      if ( (mountEntry.fileHandle == null) &&
           (fileHandle == null)
         )
        return true;                                   /* return ! */
      else
        ;
    /*end of if*/

      if (mountEntry.fileHandle == null)
        return false;                                   /* return ! */
      else
        ;
    /*end of if*/

      if ( mountEntry.fileHandle.equals( fileHandle ) )
        return true;
      else
        return false;
    /*end of if */

    }
/*end of equals*/


  public
  String

  getDirPath()
/*****************************************************************************/
/* Function name: getDirPath                                                 */
/* File name: Entry.java                                                     */
/* Purpose: to return dirPath of this Entry                                  */
/* Parameters:                                                               */
/* Returns: the dirPath string or null                                       */
/* Last modified: 21.06.99                                                   */
/* Author: Victor Klimov, 1999                                               */
/*****************************************************************************/
    {
                                                            /* getDirPath */
/*---------------------------------------------------------------------------*/

      if (fileHandle == null)
        return null;                                   /* return ! */
      else
        return fileHandle.getDirPath();
    /*end of if*/

    }
/*end of getDirPath*/


  public
  String

  getHostName()
/*****************************************************************************/
/* Function name: getHostName                                                */
/* File name: Entry.java                                                     */
/* Purpose: to return hostName of this Entry                                 */
/* Parameters:                                                               */
/* Returns: the hostName string or null                                      */
/* Last modified: 21.06.99                                                   */
/* Author: Victor Klimov, 1999                                               */
/*****************************************************************************/
    {
                                                            /* getHostName */
/*---------------------------------------------------------------------------*/

      if (fileHandle == null)
        return null;                                   /* return ! */
      else
        return fileHandle.getHostName();
    /*end of if*/

    }
/*end of getHostName*/


  public
  Fhandle

  getFileHandle()
/*****************************************************************************/
/* Function name: getFileHandle                                              */
/* File name: Entry.java                                                     */
/* Purpose: to return fileHandle of this Entry                               */
/* Parameters:                                                               */
/* Returns: the fileHandle string                                            */
/* Last modified: 15.06.99                                                   */
/* Author: Victor Klimov, 1999                                               */
/*****************************************************************************/
    {
                                                            /* getFileHandle */
/*---------------------------------------------------------------------------*/

      return fileHandle;

    }
/*end of getFileHandle*/


} /* end of Entry class */

/*****************************************************************************/
/* End of file: Entry.java                                                   */
/*****************************************************************************/
